#! /usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import copy
from random import randint
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

class PyQtGame(QWidget):
	def __init__(self):
		super(PyQtGame,self).__init__()
		self.randomInit()
		self.colors = {
			0:QColor(0xcdc1b4),
			2:QColor(0xfff9db),
			8:QColor(0xffd43b), # Changer couleur
			4:QColor(0xffec99), # Changer couleur
			16:QColor(0xfab005),
			32:QColor(0xf08c00),
			64:QColor(0xf76707),
			128:QColor(0xd9480f),
			256:QColor(0xf03e3e),
			512:QColor(0xc92a2a),
			1024:QColor(0xedc53f), # Changer couleur jusque fin
			2048:QColor(0xedc22e),
			4096:QColor(0x21170c),
			8192:QColor(0x21170c),
			16384:QColor(0x21170c),
			32768:QColor(0x21170c),
			65536:QColor(0x21170c),
			131072:QColor(0x21170c),
		}
		self.best = 0
		self.initUI()

# Initialisation de la grille
	def randomInit(self) :
		self.blocs = [[0]*4 for i in range(4)]
		for m in range(randint(2,2)) :
			self.createColor(8)
		self.score = 0
		self.overed = 0

# Taux d'apparition des blocs 2 / 4
	def randomColor(self,taux) :
		i = randint(0,taux)
		if (i < taux) :
			return 2
		else :
			return 4

# Apparition d'un nouveau bloc de 2 ou de 4
	def createColor(self,num) :
		while 1 :
			i = randint(0,3)
			j = randint(0,3)
			if self.blocs[i][j] == 0 :
				self.blocs[i][j] = self.randomColor(num)
				break
			else :
				continue

# Grille pleine
	def isFull(self) :
		for i in range(3):
			if 0 in self.blocs[i] :
				return 0
				break
		else :
			return 1

# Partie terminée
	def isOverd(self) :
		self.rowOver = 0
		self.colOver = 0
		self.colMove('right',False)
		self.rowMove('down',False)
		if self.rowOver and self.colOver and self.isFull() :
			self.gameOver()
		if self.score > self.best :
			self.best = self.score

# Définition de la fenêtre
	def initUI(self):
		#Taille fixe
		self.setFixedSize(350,400)
		self.center()
		self.setWindowTitle("Unfair 2048")
		self.show()

	def center(self):
		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

# Attribution des touches
	def keyPressEvent(self,e) :
		#Recommencer = Echap
		if e.key() == Qt.Key_Escape :
			self.resetGame()
		elif e.key() == Qt.Key_Up :
			self.rowMove('up',True)
		elif e.key() == Qt.Key_Down :
			self.rowMove('down',True)
		elif e.key() == Qt.Key_Left :
			self.colMove('left',True)
		elif e.key() == Qt.Key_Right :
			self.colMove('right',True)
		self.isOverd()

# Recommencer la partie (nouvelle grille vide)
	def resetGame(self) :
		self.randomInit()
		self.update()

# Position du pointeur lors du dernier clic
	def mousePressEvent(self,e) :
		self.lastPoint=e.pos()

# Bouton pour recommencer la partie(avec les bonnes coordonnées)
	def mouseReleaseEvent(self,e) :
		self.resetRect = QRect(240,15,80,60)
		if self.resetRect.contains(e.pos().x(),e.pos().y()) and self.resetRect.contains(self.lastPoint.x(),self.lastPoint.y()):
			if QMessageBox.question(self,'Message',"Êtes-vous sûr de vouloir recommencer ?",QMessageBox.Ok|QMessageBox.Cancel,QMessageBox.Cancel)==QMessageBox.Ok:
				self.resetGame()
		else :
			dx = e.pos().x()-self.lastPoint.x()
			dy = e.pos().y()-self.lastPoint.y()
			if abs(dx)>abs(dy) and abs(dx)>10:
				if dx > 0 :
					self.colMove('right',True)
				else :
					self.colMove('left',True)
			elif abs(dy) > 10 :
				if dy > 0 :
					self.rowMove('down',True)
				else :
					self.rowMove('up',True)

# Déplacement des lignes (haut / bas)
	def rowMove(self,direction,temp) :
		prevBloc = copy.deepcopy(self.blocs)
		for i in range(4) :
			premNumPos = -1
			premZeroPos = -1
			for j in range(4) :
				if direction == 'down':
					bigEnd = 3-j
				else :
					bigEnd = j
				if self.blocs[bigEnd][i]!=0 :
					if premNumPos==-1 and premZeroPos==-1:
						premNumPos = bigEnd
						premNumValue = self.blocs[bigEnd][i]
					elif premNumPos==-1 and premZeroPos!=-1:
						premNumPos = premZeroPos
						premNumValue = self.blocs[bigEnd][i]
						self.blocs[premZeroPos][i] = premNumValue
						self.blocs[bigEnd][i] =0
						if direction=='down' :
							premZeroPos -=1
						else :
							premZeroPos+=1
					elif premNumPos!=-1 and premZeroPos!=-1:
						if self.blocs[bigEnd][i] == premNumValue:
							premNumValue*=2
							if temp:
								self.score+=premNumValue
							self.blocs[premNumPos][i] = premNumValue
							self.blocs[bigEnd][i]=0
							premNumPos = -1
						else :
							if direction == 'down' :
								premNumPos-=1
							else :
								premNumPos+=1
							premNumValue = self.blocs[bigEnd][i]
							self.blocs[premNumPos][i]=premNumValue
							self.blocs[bigEnd][i]=0
							if direction =='down' :
								premZeroPos-=1
							else :
								premZeroPos+=1
					else:
						if self.blocs[bigEnd][i] == premNumValue:
							premNumValue*=2
							if temp:
								self.score+=premNumValue
							self.blocs[premNumPos][i] = premNumValue
							self.blocs[bigEnd][i]=0
							if direction == 'down' :
								premZeroPos = premNumPos-1
							else :
								premZeroPos = premNumPos+1
							premNumPos = -1
						else :
							if direction == 'down' :
								premNumPos-=1
							else :
								premNumPos+=1
							premNumValue = self.blocs[bigEnd][i]
				else:
					if premZeroPos ==-1:
						premZeroPos = bigEnd
		if temp :
			if prevBloc != self.blocs and not self.isFull():
				self.createColor(9)
			self.update()
		else :
			if prevBloc == self.blocs :
				self.rowOver = 1
			self.blocs = copy.deepcopy(prevBloc)

# Déplacement des colonnes (gauche / droite)
	def colMove(self,direction,temp):
		prevBloc = copy.deepcopy(self.blocs)
		for i in range(4):
			premNumPos = -1
			premZeroPos = -1
			for j in range(4):
				if direction == 'right':
					bigEnd = 3-j
				else:
					bigEnd = j
				if self.blocs[i][bigEnd]!=0:
					if premNumPos==-1 and premZeroPos==-1:
						premNumPos = bigEnd
						premNumValue = self.blocs[i][bigEnd]
					elif premNumPos==-1 and premZeroPos!=-1:
						premNumPos = premZeroPos
						premNumValue = self.blocs[i][bigEnd]
						self.blocs[i][premZeroPos] = premNumValue
						self.blocs[i][bigEnd] =0
						if direction=='right' :
							premZeroPos -=1
						else :
							premZeroPos+=1
					elif premNumPos!=-1 and premZeroPos!=-1:
						if self.blocs[i][bigEnd] == premNumValue:
							premNumValue*=2
							if temp:
								self.score+=premNumValue
							self.blocs[i][premNumPos] = premNumValue
							self.blocs[i][bigEnd]=0
							premNumPos = -1
						else :
							if direction == 'right' :
								premNumPos-=1
							else :
								premNumPos+=1
							premNumValue = self.blocs[i][bigEnd]
							self.blocs[i][premNumPos]=premNumValue
							self.blocs[i][bigEnd]=0
							if direction =='right' :
								premZeroPos-=1
							else :
								premZeroPos+=1
					else:
						if self.blocs[i][bigEnd] == premNumValue:
							premNumValue*=2
							if temp:
								self.score+=premNumValue
							self.blocs[i][premNumPos] = premNumValue
							self.blocs[i][bigEnd]=0
							if direction == 'right' :
								premZeroPos = premNumPos-1
							else :
								premZeroPos = premNumPos+1
							premNumPos = -1
						else :
							if direction == 'right' :
								premNumPos-=1
							else :
								premNumPos+=1
							premNumValue = self.blocs[i][bigEnd]
				else:
					if premZeroPos ==-1:
						premZeroPos = bigEnd
		if temp :
			if prevBloc != self.blocs and not self.isFull():
				self.createColor(9)
			self.update()
		else :
			if prevBloc == self.blocs :
				self.colOver = 1
			self.blocs=copy.deepcopy(prevBloc)

# Grille + Fenêtre + Score + Meilleur score + Reset
	def paintEvent(self,e) :
		painter = QPainter(self)
		painter.setPen(Qt.NoPen)
		painter.setBrush(QBrush(QColor(0xbbada0))) # Couleur fenêtre
		painter.drawRect(self.rect())
		painter.setBrush(QBrush(QColor(0x776e65))) # Couleur boutons score / reset
		painter.drawRoundedRect(QRect(20,15,80,60),5,5)
		painter.setFont(QFont("Century Gothic",12))
		painter.setPen(QColor(0xffffff)) # Couleur police texte "score"
		painter.drawText(QRectF(QRect(20,20,80,60)),"SCORE",QTextOption(Qt.AlignHCenter))
		painter.setFont(QFont("Century Gothic",18))
		painter.setPen(QColor(0xffffff)) # Couleur police nombre score
		painter.drawText(QRectF(QRect(20,15,80,55)),str(self.score),QTextOption(Qt.AlignHCenter|Qt.AlignBottom))
		painter.setPen(Qt.NoPen)
		painter.drawRoundedRect(QRect(130,15,80,60),5,5)
		painter.setFont(QFont("Century Gothic",12))
		painter.setPen(QColor(0xffffff)) # Couleur police texte "best"
		painter.drawText(QRectF(QRect(130,20,80,60)),"BEST",QTextOption(Qt.AlignHCenter))
		painter.setFont(QFont("Century Gothic",18))
		painter.setPen(QColor(0xffffff)) # Couleur police nombre best
		painter.drawText(QRectF(QRect(130,15,80,55)),str(self.best),QTextOption(Qt.AlignHCenter|Qt.AlignBottom))
		painter.setPen(Qt.NoPen)
		painter.drawRoundedRect(QRect(240,15,80,60),5,5)
		painter.setFont(QFont("Century Gothic",16))
		painter.setPen(QColor(0xffffff)) # Couleur police texte reset
		painter.drawText(QRectF(QRect(240,15,80,60)),"RESET",QTextOption(Qt.AlignHCenter|Qt.AlignVCenter))
		painter.setPen(Qt.NoPen)
		self.drawRectangles(painter)

#Dessins des blocs /!\ A REVOIR /!\
	def drawRectangles(self,painter):
		for i in range(4):
			for j in range(4):
				painter.setFont(QFont("Century Gothic",14,10))
				painter.setBrush(self.colors[self.blocs[i][j]])
				painter.drawRoundedRect(QRect(20+j*80,90+i*80,60,60),10,10)
				if self.blocs[i][j] != 0:
					if self.blocs[i][j]<15:
						painter.setPen(QColor(0x646464)) # Couleur police chiffre bloc < 16
					else :
						painter.setPen(QColor(0xffffff))# Couleur police chiffre bloc >= 16
					painter.drawText(QRectF(QRect(20+j*80,90+i*80,60,60)),str(self.blocs[i][j]),QTextOption(Qt.AlignHCenter|Qt.AlignVCenter))
					painter.setPen(Qt.NoPen)

#Partie perdue
	def gameOver(self):
		if QMessageBox.question(self,'Message',"<center><b>Game Over</b></center> <br> Voulez-vous recommencer ?",QMessageBox.Yes|QMessageBox.No,QMessageBox.Yes)==QMessageBox.Yes:
			self.resetGame()
		else:
			self.close()

if __name__ == '__main__':
	app = QApplication(sys.argv)
	game = PyQtGame()
app.exec_()
